/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package org.openoffice.sdk.gui;

import com.sun.star.awt.XWindow;
import com.sun.star.awt.XWindowPeer;
import com.sun.star.beans.PropertyValue;
import com.sun.star.frame.FrameSearchFlag;
import com.sun.star.frame.XComponentLoader;
import com.sun.star.frame.XDesktop;
import com.sun.star.frame.XDispatch;
import com.sun.star.frame.XDispatchHelper;
import com.sun.star.frame.XDispatchProvider;
import com.sun.star.frame.XFrame;
import com.sun.star.frame.XModuleManager;
import com.sun.star.graphic.XGraphic;
import com.sun.star.graphic.XGraphicProvider;
import com.sun.star.lang.XComponent;
import com.sun.star.uno.AnyConverter;
import com.sun.star.uno.UnoRuntime;
import com.sun.star.uno.XComponentContext;
import com.sun.star.uno.XInterface;
import com.sun.star.util.XURLTransformer;

/**
 *
 * @author arielch
 */
public class Helper {
    
    //*************************************************************************
    
    
    public static boolean DEBUG = true;

    public static void debug(String string) {
        if (DEBUG)
                System.out.println(string);
    }
    
    public static void debug(String sFormat, Object... a) {
        if (DEBUG)
                System.out.printf(sFormat, a);
    }
    
    
    //*************************************************************************
    
    public static Object createUnoService(
                                                XComponentContext xContext,
                                                String sServiceName){
        Object oService = null;
        try {
            oService = xContext.getServiceManager().createInstanceWithContext(
                    sServiceName, xContext);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return oService;
    }

    public static XInterface createUnoService(
                                                XComponentContext xContext,
                                                String sServiceName,
                                                Class xInterface){
        XInterface xService = null;
        try {
            Object oInstance = createUnoService(xContext, sServiceName);
            if (oInstance != null) {
                xService = (XInterface) AnyConverter.toObject(xInterface, oInstance);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return xService;
    }

    public static Object createUnoService(
                                                XComponentContext xContext,
                                                String sServiceName,
                                                Object[] aArguments){
        Object oService = null;
        try {
            oService = xContext.getServiceManager().createInstanceWithArgumentsAndContext(
                    sServiceName, aArguments, xContext);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return oService;
    }

    public static XInterface createUnoService(
                                                XComponentContext xContext,
                                                String sServiceName,
                                                Object[] aArguments,
                                                Class xInterface){
        XInterface xService = null;
        try {
            Object oInstance = createUnoService(xContext, sServiceName, aArguments);
            if (oInstance != null) {
                xService = (XInterface) AnyConverter.toObject(xInterface, oInstance);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return xService;
    }
    
    //*************************************************************************
    
    /**
     * PROBLEM: a method may return an Any holding a void (NULL),
     * and converting the Any will throw an exception.
     * <p>
     * Checking if not is NULL will NOT work: 
     * it DOES return an Any (only that it holds a void)
     * <pre>
     * Object oAnyValue = xPropertySet.getPropertyValue("PropertyName");
     * boolean bIsNull = (oAnyValue == null);
     * </pre>
     * bIsNull may be false, but AnyConvert.toXxxx(oAnyValue) will throw an 
     * IllegalArgumentException if this Any holds a void (null)
     * <p>
     * So we must check first if the Any is void, and then convert it.
     * 
     * @param aAny
     * @return
     */
    public static Object convertAnyToType(Class aClass, Object aAny) {
        Object conv = null;
        try {
            if (aAny != null && !AnyConverter.isVoid(aAny)) {
                conv = AnyConverter.toObject(aClass, aAny);
            }
        } catch (com.sun.star.lang.IllegalArgumentException ex) {
            ex.printStackTrace();
        }
        return conv;
    }
    
    
    public static XDesktop getDesktop(XComponentContext xContext) {
        return (XDesktop) createUnoService(
                xContext, "com.sun.star.frame.Desktop", XDesktop.class);
    }
    
    
    //*************************************************************************
    
    
    
    public static XComponent createNewDoc(XComponentContext xContext, String sType) {
        return createNewDoc(xContext, new PropertyValue[0], sType);
    }

    public static XComponent createNewDoc(
                                            XComponentContext xContext,
                                            PropertyValue[] aMediaDescriptor,
                                            String sType) {
        XComponent xComponent = null;
        try {
            XComponentLoader xDesktopLoader = (XComponentLoader) createUnoService(
                xContext, "com.sun.star.frame.Desktop", XComponentLoader.class );
            xComponent = (XComponent) UnoRuntime.queryInterface(
                    XComponent.class,
                    xDesktopLoader.loadComponentFromURL(
                    "private:factory/" + sType, "_default",
                    FrameSearchFlag.ALL, aMediaDescriptor ) );
        } catch (Exception ex) {
            ex.printStackTrace();
        }
        return xComponent;
    }
    
    /**
     * Returns a full URL for a relative path
     * @param xContext
     * @param sRelativePath     for example 
     *                          "mi_text.odt", "Dialogs/mydialog.xdl"
     *                          
     * @return  a full URL for a relative path
     */
    public static String getURLFromRelativePath(
            XComponentContext xContext, String sRelativePath)
    {
        String sURL = "";
        try {
            java.io.File file = new java.io.File(sRelativePath);
            sURL = Helper.convertToURL(xContext, file.getPath(), file.getAbsolutePath());
        } catch (Exception ex) {
            ex.printStackTrace();
        }
        return sURL;        
    }

    /**
     * Converts a system path into an URL using OOo API
     * @param sBase
     * @param sSystemPath
     * @return
     */
    public static String convertToURL(
                                        XComponentContext xContext,
                                        String sBase,
                                        String sSystemPath) {
        String sURL = null;
        try {
            com.sun.star.ucb.XFileIdentifierConverter xFileConverter =
                    (com.sun.star.ucb.XFileIdentifierConverter) UnoRuntime.queryInterface(
                    com.sun.star.ucb.XFileIdentifierConverter.class,
                    xContext.getServiceManager().createInstanceWithContext(
                    "com.sun.star.ucb.FileContentProvider", xContext));
            sURL = xFileConverter.getFileURLFromSystemPath(
                    sBase, sSystemPath);
        } catch (java.lang.Exception e) {
            e.printStackTrace();
        } finally {
            return sURL;
        }
    }

    /**
     * 
     * @param xContext
     * @param aURL
     * @return
     */
    public static boolean checkFile(XComponentContext xContext, String aURL) {
        boolean bExists = false;
        try {
            com.sun.star.ucb.XSimpleFileAccess xSFA =
                    (com.sun.star.ucb.XSimpleFileAccess) UnoRuntime.queryInterface(
                    com.sun.star.ucb.XSimpleFileAccess.class,
                    xContext.getServiceManager().createInstanceWithContext(
                    "com.sun.star.ucb.SimpleFileAccess", xContext));
            bExists = xSFA.exists(aURL) && !xSFA.isFolder(aURL);
        } catch (com.sun.star.ucb.CommandAbortedException ex) {
            ex.printStackTrace();
        } catch (com.sun.star.uno.Exception ex) {
            ex.printStackTrace();
        } catch (java.lang.Exception ex) {
            ex.printStackTrace();
        }
        return bExists;
    }

    /**
     * Provides an URL to the root of an installed package.
     * 
     * @param xContext              XComponentContext
     * @param sExtensionIdentifier  the unique identifier of an extension.
     * @return          The service looks for an installed package with the 
     * given id and returns the URL to the root of the package. 
     * If the service can not find a matching package, an emty string will 
     * be returned.
     */
    public static String getExtensionLocation(
                                                XComponentContext xContext,
                                                String sExtensionIdentifier) 
    {
        String sPath = "";
        String sSingleton =
                "/singletons/com.sun.star.deployment.PackageInformationProvider";
        try {
            com.sun.star.deployment.XPackageInformationProvider 
                    xPackageInformationProvider =
                    (com.sun.star.deployment.XPackageInformationProvider) 
                    UnoRuntime.queryInterface(
                    com.sun.star.deployment.XPackageInformationProvider.class,
                    xContext.getValueByName(sSingleton));
            sPath = xPackageInformationProvider.getPackageLocation(
                    sExtensionIdentifier);

        } catch (java.lang.Exception ex) {
            ex.printStackTrace();
        } finally {
            return sPath;
        }
    }

    /**
     * Executes an abitrary system command.
     * 
     * @param xContext          XComponentContext
     * @param sSysPath          Specifies the command to execute. 
     *                          This may be an executable file or a document 
     * which is registered with an application on a specific platform, so that 
     * the platform knows what application to launch for that document. 
     * If the command specifies a path to an executable, etc, this has to be 
     * a system specific path.
     * 
     * @throws java.lang.Exception
     */
    public static void executeSystemCommand(
            XComponentContext xContext, String sSysPath) throws Exception 
    {

        com.sun.star.system.XSystemShellExecute xSystemShellExecute =
                (com.sun.star.system.XSystemShellExecute) UnoRuntime.queryInterface(
                com.sun.star.system.XSystemShellExecute.class,
                xContext.getServiceManager().createInstanceWithContext(
                "com.sun.star.system.SystemShellExecute", xContext));
        xSystemShellExecute.execute(sSysPath, "",
                com.sun.star.system.SystemShellExecuteFlags.DEFAULTS);

    }
    public static void executeSystemCommand(
            XComponentContext xContext, String sSysPath, String aParams) throws Exception 
    {

        com.sun.star.system.XSystemShellExecute xSystemShellExecute =
                (com.sun.star.system.XSystemShellExecute) UnoRuntime.queryInterface(
                com.sun.star.system.XSystemShellExecute.class,
                xContext.getServiceManager().createInstanceWithContext(
                "com.sun.star.system.SystemShellExecute", xContext));
        xSystemShellExecute.execute(sSysPath, aParams,
                com.sun.star.system.SystemShellExecuteFlags.DEFAULTS);

    }

    /**
     * Returns the value for setting a color property of an object, as an Integer.
     * 
     * @param r     the <b>red</b> component
     * @param g     the <b>green</b> component
     * @param b     the <b>blue</b> component
     * @return      an Integer with the color value
     * @throws com.sun.star.lang.IllegalArgumentException 
     *              if any of the color components is &lt; 0 or &gt; 255
     */
    public static Integer getRGBColor(int r, int g, int b)
            throws com.sun.star.lang.IllegalArgumentException {
        boolean isRangeBad = false;
        String sErrorMessage = "";

        if (r < 0 || r > 255) {
            isRangeBad = true;
            sErrorMessage += " red component";
        }
        if (g < 0 || g > 255) {
            isRangeBad = true;
            sErrorMessage += " green component";
        }
        if (b < 0 || b > 255) {
            isRangeBad = true;
            sErrorMessage += " blue component";
        }

        if (isRangeBad) {
            throw new com.sun.star.lang.IllegalArgumentException(
                    "Color component outside the legal range: " + sErrorMessage);
        } else {
            int intValue = (b | g << 8) | (r << 16);
            return new Integer(intValue);
        }
    }

    public static String createUniqueNameForContainerElement(
                        com.sun.star.container.XNameAccess xElementContainer,
                        String elementName)
    {
        boolean bHasByName = true;
        int i = 1;
        String uniqueName = elementName;
        while (bHasByName) {
            bHasByName = xElementContainer.hasByName(elementName);
            if (bHasByName) {
                elementName = uniqueName + Integer.toString(i++);
            }
        }
        return elementName;
    }

    /**************************************************************************
     * 
     * 
     * 
     */
    /**
     * 
     * @param sName
     * @param aValue
     * @return
     */
    public static com.sun.star.beans.PropertyValue createProperty(
                                                            String sName,
                                                            Object aValue) 
    {
        com.sun.star.beans.PropertyValue aPropertyValue =
                new com.sun.star.beans.PropertyValue();
        aPropertyValue.Name = sName;
        aPropertyValue.Value = aValue;
        return aPropertyValue;
    }

    public static void setUnoPropertyValue(
            Object oUnoObject, String sPropertyName, Object aPropertyValue) 
    {
        try {
            com.sun.star.beans.XPropertySet xPropertySet =
                    (com.sun.star.beans.XPropertySet) UnoRuntime.queryInterface(
                    com.sun.star.beans.XPropertySet.class, oUnoObject);
            if (xPropertySet.getPropertySetInfo().
                    hasPropertyByName(sPropertyName)) {
                xPropertySet.setPropertyValue(sPropertyName, aPropertyValue);
            } else {
                throw new java.lang.IllegalArgumentException(
                        "No such Property: '" + sPropertyName + "'");
            }
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    public static void setUnoPropertyValues(
            Object oUnoObject, String[] sPropertyNames, Object[] aPropertyValues) 
    {
        try {
            com.sun.star.beans.XMultiPropertySet xMultiPropertySet =
                    (com.sun.star.beans.XMultiPropertySet) UnoRuntime.queryInterface(
                    com.sun.star.beans.XMultiPropertySet.class, oUnoObject);
            if (xMultiPropertySet != null) {
                xMultiPropertySet.setPropertyValues(sPropertyNames, aPropertyValues);
            } else {
                for (int i = 0; i < sPropertyNames.length; i++) {
                    setUnoPropertyValue(oUnoObject, sPropertyNames[i], aPropertyValues[i]);
                }
            }
        } catch (Exception ex) {
            ex.printStackTrace(System.out);
        }
    }

    
    public static Object getUnoPropertyValue(
            Object oUnoObject, String sPropertyName, java.lang.Class xClass) {
        try {
            if (oUnoObject != null) {
                com.sun.star.beans.XPropertySet xPropertySet = 
                        (com.sun.star.beans.XPropertySet) UnoRuntime.queryInterface(
                        com.sun.star.beans.XPropertySet.class, oUnoObject);
                Object oObject = xPropertySet.getPropertyValue(sPropertyName);
                if (AnyConverter.isVoid(oObject)) {
                    return null;
                } else {
                    return com.sun.star.uno.AnyConverter.toObject(
                            new com.sun.star.uno.Type(xClass), oObject);
                }
            }
            return null;
        } catch (Exception ex) {
            ex.printStackTrace(System.out);
            return null;
        }
    }

    public static Object getUnoPropertyValue(
            Object oUnoObject, String sPropertyName) {
        Object oObject = null;
        try {
            if (oUnoObject != null) {
                com.sun.star.beans.XPropertySet xPropertySet = 
                        (com.sun.star.beans.XPropertySet) UnoRuntime.queryInterface(
                        com.sun.star.beans.XPropertySet.class, oUnoObject);
                if (xPropertySet.getPropertySetInfo()
                        .hasPropertyByName(sPropertyName))
                {
                    oObject = xPropertySet.getPropertyValue(sPropertyName);
                } else {
                    throw new java.lang.IllegalArgumentException(
                        "No such Property: '" + sPropertyName + "'");
                }
            }
        } catch (Exception ex) {
            ex.printStackTrace();
        } finally {
            return oObject;
        }
    }
    
    /***************************************************************************
     * 
     * 
     */
    /**
     * The service <code>com.sun.star.graphic.GraphicProvider</code> is the 
     * central service of the Graphic API that gives access to graphics of any kind.<br>
     * This service allows to load graphics from and to store graphics to any location.<br>
     * The one and only interface that has to be implemented is the 
     * <code>XGraphicProvider</code> interface, that exposes the necessary methods 
     * for loading and storing the graphic contents and descriptors
     * 
     * @param xContext      <code>XComponentContext</code>
     * @return              <code>XGraphicProvider</code> interface, 
     *                      that exposes the necessary methods for loading and 
     *                      storing the graphic contents and descriptors
     */
    public static XGraphicProvider getXGraphicProvider(XComponentContext xContext){
        XGraphicProvider xGraphicProvider = null;
        try {
            xGraphicProvider = (XGraphicProvider) UnoRuntime.queryInterface(
                    XGraphicProvider.class,
                    xContext.getServiceManager().createInstanceWithContext(
                    "com.sun.star.graphic.GraphicProvider", xContext));
        } catch (Exception ex) {
            ex.printStackTrace();
        }
        return xGraphicProvider;
    }

    /**
     * 
     * @param xContext      <code>XComponentContext</code>
     * @param sURL          Property that describes the location of the source 
     *                      or target of the graphic as URL. <br>
     * In addition to the normal protocols like file:// or http:// 
     * you can use private URL's as follows to get access to graphics 
     * lying inside the resource system within an Office context: 
     * <ul>
     * <li>"private:resource/projectshortname/bitmap/12345"</li>
     * <li>"private:resource/projectshortname/bitmapex/12345"</li>
     * <li>"private:resource/projectshortname/image/12345"</li>
     * </ul>
     * Yet more, you can access graphics in the application-wide image repository 
     * by specifying URLs of the form 
     * private:graphicrepository/&lt;path_in_repository&gt;.
     * <ul>
     * <li>"private:graphicrepository/res/foldercl.png"</li>
     * <li>"private:graphicrepository/res/folderop.png"</li>
     * <li>"private:graphicrepository/sfx2/res/hlpbookclosed.png"</li>
     * </ul>
     * @return       a <code>XGraphic</code> interface that holds the 
     *              graphic content after loading the graphic
     */
    public static XGraphic getGraphicFromURL(
            XComponentContext xContext, String sURL)
    {
        XGraphic xGraphic = null;
        try {
            XGraphicProvider xGraphicProvider = getXGraphicProvider(xContext);
            PropertyValue[] aMediaDescriptor = new PropertyValue[1];
            aMediaDescriptor[0] = createProperty("URL", sURL);
            xGraphic = xGraphicProvider.queryGraphic(aMediaDescriptor);
        } catch (Exception ex) {
            ex.printStackTrace();
        }
        return xGraphic;
    }
    
    /**
     * Stores a graphic content at the specified location 
     * 
     * @param xContext      <code>XComponentContext</code>
     * @param sURL
     * @param sMimeType     This property is used for storing graphics and 
     *                      describes the format into which the graphic is to be converted <br>
     * At the moment, the following mime types are supported for storing graphics: 
     * <ul>
     * <li>image/bmp</li><li>image/gif</li><li>image/jpeg</li><li>image/png</li>
     * <li>image/tiff</li><li>image/svg+xml</li><li>image/x-cmu-raster</li>
     * <li>image/x-emf</li><li>image/x-eps</li><li>image/x-met</li><li>image/x-pict</li>
     * <li>image/x-portable-bitmap</li><li>image/x-portable-pixmap</li><li>image/x-wmf</li>
     * <li>image/x-svm</li><li>image/x-xpixmap</li><li>image/x-vclgraphic</li>
     * </ul>
     * @param aFilterData   Additional properties that will be passed to the 
     *                      appropriate filter module.<br>Please see<br>
     * <a href="http://api.openoffice.org/servlets/ReadMsg?listName=dev&msgNo=7101">http://api.openoffice.org/servlets/ReadMsg?listName=dev&msgNo=7101</a><br>
     * <a href="http://api.openoffice.org/servlets/ReadMsg?listName=dev&msgNo=12959">http://api.openoffice.org/servlets/ReadMsg?listName=dev&msgNo=12959</a>
     * 
     * And see 
     * svx/source/unodraw/UnoGraphicExporter.cxx
     * binfilter/bf_svx/source/unodraw/svx_UnoGraphicExporter.cxx
     * <blockquote>
     * <pre>
     * PropertyValue[] aFilterData = new PropertyValue[5];
     * aFilterData[0] = createProperty( "PixelWidth", 800);
     * aFilterData[1] = createProperty( "PixelHeight", 600);
     * aFilterData[2] = createProperty( "LogicalWidth", 100);
     * aFilterData[3] = createProperty( "LogicalHeight", 100);
     * aFilterData[4] = createProperty( "Quality", 50);
     * </pre>
     * </blockquote>
     * 
     * 
     * @return              true if the graphic could be stored
     */
    public static boolean storeGraphicToURL(
                                XComponentContext xContext, XGraphic xGraphic, String sURL, 
                                String sMimeType, PropertyValue[] aFilterData)
    {
        boolean bStored = false;
        try {
            XGraphicProvider xGraphicProvider = getXGraphicProvider(xContext);
            
            int iBound = (aFilterData==null) ? 2 : 3;
            PropertyValue[] aMediaDescriptor = new PropertyValue[iBound];
            aMediaDescriptor[0] = createProperty("URL", sURL);
            aMediaDescriptor[1] = createProperty("MimeType", sURL);
            
            if (aFilterData!=null) {
                aMediaDescriptor[2] = createProperty("FilterData", aFilterData);
            }
            xGraphicProvider.storeGraphic(xGraphic, aMediaDescriptor);
            bStored = true;java.text.NumberFormat a;
            
        } catch (Exception ex) {
            ex.printStackTrace();
        }
        return bStored;
    }
    
    
    //**************************************************************************
    
    /**
     * Queries XWindow from XWindow
     * @param xControl
     * @return
     */
    public static XWindowPeer getWindowPeer(XWindow xWindow){
        return (XWindowPeer) UnoRuntime.queryInterface(XWindowPeer.class, xWindow);
    }
    
    
    //*************************************************************************
    
    public static String identifyModule( XComponentContext xContext, Object xIfc ){
        try {
            XModuleManager xModuleManager = (XModuleManager) UnoRuntime.queryInterface(
                    XModuleManager.class, xContext.getServiceManager().createInstanceWithContext(
                    "com.sun.star.frame.ModuleManager", xContext));
            String sModule = xModuleManager.identify(xIfc);
            return sModule;
        } catch (Exception e){
            e.printStackTrace();
        }
        return null;
    }
    
    
    //*************************************************************************
    
    public static XDispatchHelper getDispatchHelper(XComponentContext xContext){
        return (XDispatchHelper) createUnoService(xContext, 
                "com.sun.star.frame.DispatchHelper", XDispatchHelper.class);
    }
    
    /**
     * 
     * @param xContext
     * @param aFrame
     * @param aDispatchURL
     * @param aTargetFrameName
     * @param someSearchFlags
     * @param someArgs
     * @return
     */
    public static Object dispatchFast(    
                                        XComponentContext xContext,
                                        XFrame aFrame,
                                        String aDispatchURL,
                                        String aTargetFrameName,
                                        int someSearchFlags,
                                        PropertyValue[] someArgs){
        XDispatchHelper xDispatchHelper = getDispatchHelper(xContext);
        XDispatchProvider xDispatchProvider = (XDispatchProvider)
            UnoRuntime.queryInterface(XDispatchProvider.class, aFrame);
        return xDispatchHelper.executeDispatch(
                xDispatchProvider, aDispatchURL, aTargetFrameName, someSearchFlags, someArgs);       
    }
    
    /**
     * 
     * @param xContext
     * @param aFrame
     * @param aDispatchURL
     * @param aTargetFrameName
     * @return
     */
    public static Object dispatchFast(    
                                        XComponentContext xContext,
                                        XFrame aFrame,
                                        String aDispatchURL,
                                        String aTargetFrameName){
        return dispatchFast(xContext, aFrame, aDispatchURL, 
                aTargetFrameName, FrameSearchFlag.ALL, new PropertyValue[0]);       
    }
    
    /***************************************************************************/
    
    /**
     *  Retorna una referencia a XDispatch.
     *  La URL debe ser una com.sun.star.util.URL "parseada".
     */
    public static XDispatch getXDispatch(   XFrame aFrame,
                                            String aTargetFrameName,
                                            com.sun.star.util.URL oURL) {
        try {
            com.sun.star.util.URL[] oURLArray = new com.sun.star.util.URL[1];
            oURLArray[0] = oURL;
            XDispatchProvider xDispatchProvider = (XDispatchProvider)
                UnoRuntime.queryInterface(XDispatchProvider.class, aFrame);
            XDispatch xDispatch = xDispatchProvider.queryDispatch(
                    oURLArray[0], aTargetFrameName, FrameSearchFlag.ALL);
            return xDispatch;
        } catch (Exception e) {
            e.printStackTrace(System.out);
        }
        return null;
    }

    /**
     *  Method to PARSE a string URL returning a com.sun.star.util.URL
     *
     * @param xContext
     * @param sURL
     * @return
     */
    public static com.sun.star.util.URL parseURL( XComponentContext xContext, 
                                                        String sURL){
        com.sun.star.util.URL aUrl = null;
        try {
            XURLTransformer xTransformer = (XURLTransformer) createUnoService(
                    xContext, "com.sun.star.util.URLTransformer", 
                    XURLTransformer.class);
            if (xTransformer != null) {
                com.sun.star.util.URL[] oURL = new com.sun.star.util.URL[1];
                oURL[0] = new com.sun.star.util.URL();
                oURL[0].Complete = sURL;
                xTransformer.parseStrict(oURL);
                aUrl = oURL[0];
            }
        } catch (Exception e) {
            e.printStackTrace(System.err);
        }
        return aUrl;
    }
    
    public static void dispatchURL( XComponentContext xContext,
                                    XFrame xFrame,
                                    String sURL,
                                    String aTargetFrameName,
                                    PropertyValue[] someArgs) {
        com.sun.star.util.URL oURL = parseURL(xContext, sURL);
        XDispatch xDispatch = getXDispatch( xFrame, aTargetFrameName, oURL );
        if (xDispatch != null) {
            xDispatch.dispatch(oURL, someArgs);
        }
    }
    
    public static void dispatchURL( XComponentContext xContext,
                                    XFrame xFrame,
                                    String sURL,
                                    String aTargetFrameName) {
        dispatchURL(xContext, xFrame, sURL, aTargetFrameName, new PropertyValue[0]);
    }
    
    //*************************************************************************


}
