/**
 * 
 */
package org.openoffice.sdk.gui;

import com.sun.star.awt.MessageBoxButtons;
import com.sun.star.awt.PosSize;
import com.sun.star.awt.Rectangle;
import com.sun.star.awt.XControl;
import com.sun.star.awt.XControlContainer;
import com.sun.star.awt.XControlModel;
import com.sun.star.awt.XDialog;
import com.sun.star.awt.XDialogProvider2;
import com.sun.star.awt.XMessageBox;
import com.sun.star.awt.XMessageBoxFactory;
import com.sun.star.awt.XToolkit;
import com.sun.star.awt.XWindow;
import com.sun.star.awt.XWindowPeer;
import com.sun.star.beans.PropertyValue;
import com.sun.star.beans.XPropertySet;
import com.sun.star.frame.XFrame;
import com.sun.star.frame.XModel;
import com.sun.star.frame.XModuleManager;
import com.sun.star.frame.XStatusListener;
import com.sun.star.lang.XComponent;
import com.sun.star.lang.XMultiComponentFactory;
import com.sun.star.uno.XComponentContext;
import com.sun.star.lib.uno.helper.Factory;
import com.sun.star.lang.XSingleComponentFactory;
import com.sun.star.registry.XRegistryKey;
import com.sun.star.lib.uno.helper.WeakBase;
import com.sun.star.uno.AnyConverter;
import com.sun.star.uno.UnoRuntime;
import com.sun.star.util.URL;


public final class ProtocolHandlerImpl extends WeakBase
   implements com.sun.star.lang.XServiceInfo,
              com.sun.star.frame.XDispatchProvider,
              com.sun.star.frame.XDispatch,
              com.sun.star.lang.XInitialization,
              com.sun.star.awt.XDialogEventHandler
{
    private final XComponentContext m_xContext;
    private XMultiComponentFactory m_xMCF;
    /**
     * points to the frame context in which this handler runs,
     * is set in initialize()
     */
    private com.sun.star.frame.XFrame m_xFrame = null;
    private com.sun.star.frame.XModel m_xModel = null;
    private com.sun.star.lang.XComponent m_xCurrentComponent = null;
    private com.sun.star.frame.XController m_xController = null;
    private com.sun.star.frame.XModuleManager m_xModuleManager = null;
    private com.sun.star.awt.XWindow m_xContainerWindow = null;
    private com.sun.star.awt.XWindow m_xComponentWindow = null;
    private XWindowPeer m_xDlgWindowPeer = null;
    private static String m_sExtensionLocation = null;
    private static final String m_implementationName = ExtensionStrings.PROTOCOL_HANDLER_IMPL_NAME;
    private static final String[] m_serviceNames = {
        "com.sun.star.frame.ProtocolHandler" };
    

    public ProtocolHandlerImpl( XComponentContext xContext )
    {
        m_xContext = xContext;
        m_xMCF = m_xContext.getServiceManager();

        if (m_sExtensionLocation == null)
            m_sExtensionLocation = Helper.getExtensionLocation(
                xContext, ExtensionStrings.EXTENSION_ID);

        try {
            m_xModuleManager = (XModuleManager) UnoRuntime.queryInterface(
                XModuleManager.class, m_xMCF.createInstanceWithContext(
                "com.sun.star.frame.ModuleManager", m_xContext));
        } catch (com.sun.star.uno.Exception ex) {
            ex.printStackTrace();
            return;
        }

    };

    //**************************************************************************


    public static XSingleComponentFactory __getComponentFactory( String sImplementationName ) {
        XSingleComponentFactory xFactory = null;

        if ( sImplementationName.equals( m_implementationName ) )
            xFactory = Factory.createComponentFactory(ProtocolHandlerImpl.class, m_serviceNames);
        return xFactory;
    }

    public static boolean __writeRegistryServiceInfo( XRegistryKey xRegistryKey ) {
        return Factory.writeRegistryServiceInfo(m_implementationName,
                                                m_serviceNames,
                                                xRegistryKey);
    }

    //**************************************************************************

    // com.sun.star.lang.XServiceInfo:
    public String getImplementationName() {
         return m_implementationName;
    }

    public boolean supportsService( String sService ) {
        int len = m_serviceNames.length;

        for( int i=0; i < len; i++) {
            if (sService.equals(m_serviceNames[i]))
                return true;
        }
        return false;
    }

    public String[] getSupportedServiceNames() {
        return m_serviceNames;
    }

    //**************************************************************************

    // com.sun.star.lang.XInitialization:
    synchronized public void initialize(Object[] aArguments) throws com.sun.star.uno.Exception
    {
        if ( aArguments.length > 0 ) {
            m_xFrame = (com.sun.star.frame.XFrame)UnoRuntime.queryInterface(
                    com.sun.star.frame.XFrame.class, aArguments[0]);
            if ( m_xFrame != null ){
                m_xController = m_xFrame.getController();
                m_xContainerWindow = m_xFrame.getContainerWindow();
                m_xComponentWindow = m_xFrame.getComponentWindow();
                if ( m_xController != null ){
                    m_xModel = m_xController.getModel();
                    m_xCurrentComponent = (XComponent) UnoRuntime.queryInterface(
                                XComponent.class, m_xModel);
                }
            }
        }
    }

    //**************************************************************************

    // com.sun.star.frame.XDispatchProvider:
    public com.sun.star.frame.XDispatch queryDispatch(
            com.sun.star.util.URL aURL, String TargetFrameName, int SearchFlags)
    {
        XFrame xFrame = null;
        synchronized(this){
            xFrame = m_xFrame;
        }
        // if the component hasn't been initialize with an XFrame
        // return null
        if ( xFrame == null ){
            return null;
        }
        
        if ( aURL.Protocol.compareTo(ExtensionStrings.PROTOCOL_HANDLER_PROTOCOL) == 0 ) {
            if (
                    ( aURL.Path.compareTo(ExtensionStrings.PROTOCOL_HANDLER_URL_PATH) == 0 ) /*||
                    ( aURL.Path.compareTo("") == 0 ) ||
                    ( aURL.Path.compareTo("") == 0 ) ||
                    ( aURL.Path.compareTo("") == 0 ) ||
                    ( aURL.Path.compareTo("") == 0 ) ||
                    ( aURL.Path.compareTo("") == 0 )*/
                )
                return this;
        }
        return null;
    }

    // com.sun.star.frame.XDispatchProvider:
    public com.sun.star.frame.XDispatch[] queryDispatches(
            com.sun.star.frame.DispatchDescriptor[] seqDescriptors)
    {
        int nCount = seqDescriptors.length;
        com.sun.star.frame.XDispatch[] seqDispatcher =
                new com.sun.star.frame.XDispatch[seqDescriptors.length];

        for( int i=0; i < nCount; ++i ) {
            seqDispatcher[i] = queryDispatch(seqDescriptors[i].FeatureURL,
                    seqDescriptors[i].FrameName,
                    seqDescriptors[i].SearchFlags );
        }
        return seqDispatcher;
    }

    //**************************************************************************

    //  XDispatch
    public void dispatch(URL aURL, PropertyValue[] aProps) {
        if (aURL.Protocol.compareTo(ExtensionStrings.PROTOCOL_HANDLER_PROTOCOL) == 0) {
            if (aURL.Path.compareTo(ExtensionStrings.PROTOCOL_HANDLER_URL_PATH) == 0) {
                executeDlgXML();
            }
            /*if (aURL.Path.compareTo("") == 0) {
                showDispatchInfo(aURL);
            }
            if (aURL.Path.compareTo("") == 0) {
                showDispatchInfo(aURL);
            }
            if (aURL.Path.compareTo("") == 0) {
                showDispatchInfo(aURL);
            }
            if (aURL.Path.compareTo("") == 0) {
                showDispatchInfo(aURL);
            }
            if (aURL.Path.compareTo("") == 0) {
                showDispatchInfo(aURL);
            }*/
        }
    }

    public void addStatusListener(XStatusListener xStatusListener, URL aURL) {
    }

    public void removeStatusListener(XStatusListener xStatusListener, URL aURL) {
    }
    //*************************************************************************

    // com.sun.star.awt.XDialogEventHandler:
    
    public boolean callHandlerMethod(
                                        com.sun.star.awt.XDialog xDialog, 
                                        Object EventObject, 
                                        String MethodName) 
            throws com.sun.star.lang.WrappedTargetException 
    {
        XWindowPeer xWindowPeer = null;
        synchronized(this){
            xWindowPeer = m_xDlgWindowPeer;
        }

        if (MethodName.equals(ExtensionStrings.XML_DLG_HANDLER_METHOD_COPY_TEXT)) {
            copyText(xDialog, EventObject);
            showInfoBox(
                    xWindowPeer,
                    "DialogComponent",
                    "callHandlerMethod() handled \"" +
                    ExtensionStrings.XML_DLG_HANDLER_METHOD_COPY_TEXT + "\"");
            return true;
        } else if (MethodName.equals(ExtensionStrings.XML_DLG_HANDLER_METHOD_DO_IT_1)) {
            showInfoBox(
                    xWindowPeer,
                    "DialogComponent",
                    "callHandlerMethod() handled \"" +
                    ExtensionStrings.XML_DLG_HANDLER_METHOD_DO_IT_1 + "\"");
            return true;
        } else if (MethodName.equals(ExtensionStrings.XML_DLG_HANDLER_METHOD_DO_IT_2)) {
            showInfoBox(
                    xWindowPeer,
                    "DialogComponent",
                    "callHandlerMethod() handled \"" +
                    ExtensionStrings.XML_DLG_HANDLER_METHOD_DO_IT_2 + "\"");
            return true;
        } else if (MethodName.equals(ExtensionStrings.XML_DLG_HANDLER_METHOD_DO_IT_3)) {
            showInfoBox(
                    xWindowPeer,
                    "DialogComponent",
                    "callHandlerMethod() handled \"" +
                    ExtensionStrings.XML_DLG_HANDLER_METHOD_DO_IT_3 + "\"");
            return true;
        }
        return false;
    }

    public String[] getSupportedMethodNames() {
        String[] retValue = {
            ExtensionStrings.XML_DLG_HANDLER_METHOD_COPY_TEXT,
            ExtensionStrings.XML_DLG_HANDLER_METHOD_DO_IT_1,
            ExtensionStrings.XML_DLG_HANDLER_METHOD_DO_IT_2,
            ExtensionStrings.XML_DLG_HANDLER_METHOD_DO_IT_3
        };
        return retValue;
    }

    //**************************************************************************
    //**************************************************************************

    private void executeDlgXML() {
        try {
            XWindow xCompWin = null, xContWin = null, aParentWin = null;
            XModel xModel = null;
            XFrame xFrame = null;

            synchronized(this){
                xCompWin = m_xComponentWindow;
                xContWin = m_xContainerWindow;
                xFrame = m_xFrame;
                xModel = m_xModel;
            }
            XWindow aParentWindow = (xCompWin==null)
                    ? xContWin : xCompWin;

            String sDlgURL = m_sExtensionLocation + '/' +
                    ExtensionStrings.XML_DIALOG_FILE_NAME;
            if ( !Helper.checkFile(m_xContext, sDlgURL) ){
                showErrorBox(
                        Helper.getWindowPeer(aParentWindow),
                        "ERROR: missing resource",
                        "An error ocurred:\n\nImpossible to locate the dialog" +
                        "\n\t" + sDlgURL);
                return;
            }

            XDialog xDialog = createDialog(sDlgURL, xModel, xFrame, aParentWindow);

            int n = xDialog.execute();
            System.out.println("XDialog.execute() returned " + n);

        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    private XDialog createDialog(
                                String sDialogURL,
                                com.sun.star.frame.XModel xModel,
                                com.sun.star.frame.XFrame xFrame,
                                XWindow xParentWindow)
    {
        XDialog xDialog = null;
        try {
            XMultiComponentFactory xMCF = m_xContext.getServiceManager();
            Object oDialogProvider;
            // If valid we must pass the XModel when creating a DialogProvider object
            if (xModel != null) {
                Object[] args = new Object[1];
                args[0] = xModel;

                oDialogProvider = xMCF.createInstanceWithArgumentsAndContext(
                        "com.sun.star.awt.DialogProvider2", args, m_xContext);
            } else {
                oDialogProvider = xMCF.createInstanceWithContext(
                        "com.sun.star.awt.DialogProvider2", m_xContext);
            }

            XDialogProvider2 xDialogProvider = (XDialogProvider2) UnoRuntime.queryInterface(
                    XDialogProvider2.class, oDialogProvider);

            xDialog = xDialogProvider.createDialogWithHandler(sDialogURL, this);
            
            XWindow xDlgWindow = (XWindow) UnoRuntime.queryInterface(
                XWindow.class, xDialog);
            
            XControl xDlgControl = (XControl) UnoRuntime.queryInterface(
                XControl.class, xDialog);

            synchronized (this) {
                m_xDlgWindowPeer = xDlgControl.getPeer();
            }
            
            if ( xFrame != null )
                xParentWindow = xFrame.getContainerWindow();
            
            if (xParentWindow == null) {
                centerDialogOnScreen(xDlgWindow);
            } else {
                centerDialog(xDlgWindow, xParentWindow.getPosSize());
            }
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            return xDialog;
        }
    }

    public void copyText(
                            com.sun.star.awt.XDialog xDialog, 
                            Object aEventObject) 
    {
        XControlContainer xControlContainer = (XControlContainer) UnoRuntime.queryInterface(
                XControlContainer.class, xDialog);
        
        String aTextPropertyStr = "Text";
        String aText = "";
        
        XControl xTextField1Control = xControlContainer.getControl("TextField1");
        XControlModel xControlModel1 = xTextField1Control.getModel();
        
        XPropertySet xPropertySet1 = (XPropertySet) UnoRuntime.queryInterface(
                XPropertySet.class, xControlModel1);
        try {
            aText = AnyConverter.toString(
                    xPropertySet1.getPropertyValue(aTextPropertyStr));
        } catch (Exception e) {
            e.printStackTrace();
        }

        XControl xTextField2Control = xControlContainer.getControl("TextField2");
        XControlModel xControlModel2 = xTextField2Control.getModel();
        XPropertySet xPropertySet2 = (XPropertySet) UnoRuntime.queryInterface(
                XPropertySet.class, xControlModel2);
        try {
            xPropertySet2.setPropertyValue(aTextPropertyStr, aText);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
    
    //************************************************************************

    public short showMessageBox(
                                XWindowPeer xWindowPeer, 
                                String sTitle, 
                                String sMessage,
                                String sType)
    {
        short nRet = -1;
        try {
            if (xWindowPeer!=null) {
                
                XToolkit xToolkit = xWindowPeer.getToolkit();
                
                XMessageBoxFactory xMessageBoxFactory = (XMessageBoxFactory)
                        UnoRuntime.queryInterface(XMessageBoxFactory.class, xToolkit);
                
                XMessageBox xMsgBox = xMessageBoxFactory.createMessageBox(
                        xWindowPeer, new Rectangle(), sType,
                        MessageBoxButtons.BUTTONS_OK, sTitle, sMessage);
                nRet = xMsgBox.execute();

                XComponent xComponent = (XComponent) UnoRuntime.queryInterface(
                        XComponent.class, xMsgBox);
                xComponent.dispose();
            }
        } catch (Exception ex) {
            ex.printStackTrace();
        } finally {
            return nRet;
        }
    }

    public short showInfoBox(
                                XWindowPeer xWindowPeer,
                                String sTitle,
                                String sMessage)
    {
        return showMessageBox(xWindowPeer, sTitle, sMessage, "infobox");
    }

    public short showErrorBox(
                                XWindowPeer xWindowPeer,
                                String sTitle,
                                String sMessage)
    {
        return showMessageBox(xWindowPeer, sTitle, sMessage, "errorbox");
    }
    
    
    //*************************************************************************
    
    
    private void centerDialogOnScreen(XWindow xDialogWindow) {
        try {
            java.awt.Toolkit sysToolkit = java.awt.Toolkit.getDefaultToolkit();
            java.awt.Dimension screenSize = sysToolkit.getScreenSize();
            Rectangle screenPosSize = new Rectangle(0, 0, screenSize.width, screenSize.height);
            centerDialog(xDialogWindow, screenPosSize);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private void centerDialog(
                                XWindow xDialogWindow, 
                                Rectangle parentWindowPosSize) 
    {
        try {
            Rectangle CurPosSize = xDialogWindow.getPosSize();

            int WindowHeight = parentWindowPosSize.Height;
            int WindowWidth = parentWindowPosSize.Width;

            int DialogWidth = CurPosSize.Width;
            int DialogHeight = CurPosSize.Height;

            int iXPos = ((WindowWidth / 2) - (DialogWidth / 2));
            int iYPos = ((WindowHeight / 2) - (DialogHeight / 2));

            xDialogWindow.setPosSize(
                    iXPos, iYPos,
                    DialogWidth, DialogHeight,
                    PosSize.POS);

        } catch (java.lang.Exception e) {
            e.printStackTrace();
        }
    }


}
